#include "NaturalLight.h"
#include <Arduino.h>
#include <ESP8266HTTPClient.h>
#include <ArduinoJson.h>
#include <time.h>

NaturalLight::NaturalLight(char *lat, char *lon, int timezoneOffset) {
	_timezoneOffset = timezoneOffset;
	sprintf(_url, SOLAR_EVENT_URL, lat, lon);
}

String NaturalLight::getSolarEventJson() {
	HTTPClient http;
	http.begin(_url);
	int httpCode = http.GET();
	String result = "";
	if (httpCode > 0)
		result = http.getString();
	http.end();

	return result;
}

int NaturalLight::getEpoch(const char *str) {
	struct tm time;
	int year, month;
	sscanf(str, "%d-%d-%dT%d:%d:%d+00:00", &year, &month, &time.tm_mday, &time.tm_hour, &time.tm_min, &time.tm_sec);
	time.tm_year = year - 1900;
	time.tm_mon = month - 1;
	return mktime(&time);
}

int NaturalLight::adjustForTimezone(int epoch) {
	return epoch + _timezoneOffset;
}

void NaturalLight::update() {
	String json = getSolarEventJson();
	StaticJsonBuffer<1024> jsonBuffer;
	JsonObject &root = jsonBuffer.parseObject(json);

	if (!root.success())
		return;

	const char *astronomicalTwilightBegin = root["results"]["astronomical_twilight_begin"];
	const char *sunrise = root["results"]["sunrise"];
	const char *sunset = root["results"]["sunset"];
	const char *astronomicalTwilightEnd = root["results"]["astronomical_twilight_end"];

	_astronomicalTwilightBegin = adjustForTimezone(getEpoch(astronomicalTwilightBegin));
	_sunrise = adjustForTimezone(getEpoch(sunrise));
	_sunset = adjustForTimezone(getEpoch(sunset));
	_astronomicalTwilightEnd = adjustForTimezone(getEpoch(astronomicalTwilightEnd));
}